@@ -20,6 +20,7 @@
 
 #include "config.h"
 
+#include <math.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <sys/time.h>
@@ -34,40 +35,47 @@
 #include <gconf/gconf-client.h>
 
 #include "gdm-languages.h"
+#include "gdm-layouts.h"
 #include "gdm-greeter-panel.h"
 #include "gdm-clock-widget.h"
 #include "gdm-language-option-widget.h"
+#include "gdm-layout-option-widget.h"
 #include "gdm-session-option-widget.h"
-#include "gdm-a11y-preferences-dialog.h"
+#include "gdm-timer.h"
+#include "gdm-profile.h"
 
 #include "na-tray.h"
 
-#define KEY_DISABLE_A11Y_BUTTON "/apps/gdm/simple-greeter/disable-accessibility-button"
-
 #define GDM_GREETER_PANEL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_GREETER_PANEL, GdmGreeterPanelPrivate))
 
 struct GdmGreeterPanelPrivate
 {
         int                     monitor;
         GdkRectangle            geometry;
-        GtkWidget              *a11y_button;
-        GtkWidget              *a11y_dialog;
         GtkWidget              *hbox;
+        GtkWidget              *alignment;
+        GtkWidget              *option_hbox;
         GtkWidget              *hostname_label;
         GtkWidget              *clock;
         GtkWidget              *language_option_widget;
+        GtkWidget              *layout_option_widget;
         GtkWidget              *session_option_widget;
 
+        GdmTimer               *animation_timer;
+        double                  progress;
+
         char                   *default_session_name;
         char                   *default_language_name;
 };
 
 enum {
         PROP_0,
+        PROP_MONITOR
 };
 
 enum {
         LANGUAGE_SELECTED,
+        LAYOUT_SELECTED,
         SESSION_SELECTED,
         NUMBER_OF_SIGNALS
 };
@@ -80,13 +88,37 @@ static void     gdm_greeter_panel_finalize    (GObject              *object);
 
 G_DEFINE_TYPE (GdmGreeterPanel, gdm_greeter_panel, GTK_TYPE_WINDOW)
 
+static void
+gdm_greeter_panel_set_monitor (GdmGreeterPanel *panel,
+                               int              monitor)
+{
+        g_return_if_fail (GDM_IS_GREETER_PANEL (panel));
+
+        if (panel->priv->monitor == monitor) {
+                return;
+        }
+
+        panel->priv->monitor = monitor;
+
+        gtk_widget_queue_resize (GTK_WIDGET (panel));
+
+        g_object_notify (G_OBJECT (panel), "monitor");
+}
+
 static void
 gdm_greeter_panel_set_property (GObject        *object,
                                 guint           prop_id,
                                 const GValue   *value,
                                 GParamSpec     *pspec)
 {
+        GdmGreeterPanel *self;
+
+        self = GDM_GREETER_PANEL (object);
+
         switch (prop_id) {
+        case PROP_MONITOR:
+                gdm_greeter_panel_set_monitor (self, g_value_get_int (value));
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -99,7 +131,14 @@ gdm_greeter_panel_get_property (GObject        *object,
                                 GValue         *value,
                                 GParamSpec     *pspec)
 {
+        GdmGreeterPanel *self;
+
+        self = GDM_GREETER_PANEL (object);
+
         switch (prop_id) {
+        case PROP_MONITOR:
+                g_value_set_int (value, self->priv->monitor);
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -113,10 +152,14 @@ gdm_greeter_panel_constructor (GType                  type,
 {
         GdmGreeterPanel      *greeter_panel;
 
+        gdm_profile_start (NULL);
+
         greeter_panel = GDM_GREETER_PANEL (G_OBJECT_CLASS (gdm_greeter_panel_parent_class)->constructor (type,
                                                                                                          n_construct_properties,
                                                                                                          construct_properties));
 
+        gdm_profile_end (NULL);
+
         return G_OBJECT (greeter_panel);
 }
 
@@ -189,46 +232,6 @@ gdm_greeter_panel_real_unrealize (GtkWidget *widget)
         }
 }
 
-static GdkRegion *
-get_outside_region (GdmGreeterPanel *panel)
-{
-        int        i;
-        GdkRegion *region;
-
-        region = gdk_region_new ();
-        for (i = 0; i < panel->priv->monitor; i++) {
-                GdkRectangle geometry;
-
-                gdk_screen_get_monitor_geometry (GTK_WINDOW (panel)->screen,
-                                                 i,
-                                                 &geometry);
-                gdk_region_union_with_rect (region, &geometry);
-        }
-
-        return region;
-}
-
-static void
-get_monitor_geometry (GdmGreeterPanel *panel,
-                      GdkRectangle    *geometry)
-{
-        GdkRegion   *outside_region;
-        GdkRegion   *monitor_region;
-        GdkRectangle geom;
-
-        outside_region = get_outside_region (panel);
-
-        gdk_screen_get_monitor_geometry (GTK_WINDOW (panel)->screen,
-                                         panel->priv->monitor,
-                                         &geom);
-        monitor_region = gdk_region_rectangle (&geom);
-        gdk_region_subtract (monitor_region, outside_region);
-        gdk_region_destroy (outside_region);
-
-        gdk_region_get_clipbox (monitor_region, geometry);
-        gdk_region_destroy (monitor_region);
-}
-
 static void
 set_struts (GdmGreeterPanel *panel,
             int              x,
@@ -289,7 +292,9 @@ update_struts (GdmGreeterPanel *panel)
 {
         GdkRectangle geometry;
 
-        get_monitor_geometry (panel, &geometry);
+        gdk_screen_get_monitor_geometry (GTK_WINDOW (panel)->screen,
+                                         panel->priv->monitor,
+                                         &geometry);
 
         /* FIXME: assumes only one panel */
         set_struts (panel,
@@ -305,13 +310,15 @@ update_geometry (GdmGreeterPanel *panel,
 {
         GdkRectangle geometry;
 
-        get_monitor_geometry (panel, &geometry);
+        gdk_screen_get_monitor_geometry (GTK_WINDOW (panel)->screen,
+                                         panel->priv->monitor,
+                                         &geometry);
 
         panel->priv->geometry.width = geometry.width;
         panel->priv->geometry.height = requisition->height + 2 * GTK_CONTAINER (panel)->border_width;
 
         panel->priv->geometry.x = geometry.x;
-        panel->priv->geometry.y = geometry.y + geometry.height - panel->priv->geometry.height;
+        panel->priv->geometry.y = geometry.y + geometry.height - panel->priv->geometry.height + (1.0 - panel->priv->progress) * panel->priv->geometry.height;
 
 #if 0
         g_debug ("Setting geometry x:%d y:%d w:%d h:%d",
@@ -364,6 +371,39 @@ gdm_greeter_panel_real_size_request (GtkWidget      *widget,
 
         gdm_greeter_panel_move_resize_window (panel, position_changed, size_changed);
 }
+static void
+gdm_greeter_panel_real_show (GtkWidget *widget)
+{
+        GdmGreeterPanel *panel;
+        GtkSettings *settings;
+        gboolean     animations_are_enabled;
+
+        settings = gtk_settings_get_for_screen (gtk_widget_get_screen (widget));
+        g_object_get (settings, "gtk-enable-animations", &animations_are_enabled, NULL);
+
+        panel = GDM_GREETER_PANEL (widget);
+
+        if (animations_are_enabled) {
+                gdm_timer_start (panel->priv->animation_timer, 1.0);
+        } else {
+                panel->priv->progress = 1.0;
+        }
+
+        GTK_WIDGET_CLASS (gdm_greeter_panel_parent_class)->show (widget);
+}
+
+static void
+gdm_greeter_panel_real_hide (GtkWidget *widget)
+{
+        GdmGreeterPanel *panel;
+
+        panel = GDM_GREETER_PANEL (widget);
+
+        gdm_timer_stop (panel->priv->animation_timer);
+        panel->priv->progress = 0.0;
+
+        GTK_WIDGET_CLASS (gdm_greeter_panel_parent_class)->hide (widget);
+}
 
 static void
 gdm_greeter_panel_class_init (GdmGreeterPanelClass *klass)
@@ -380,6 +420,9 @@ gdm_greeter_panel_class_init (GdmGreeterPanelClass *klass)
         widget_class->realize = gdm_greeter_panel_real_realize;
         widget_class->unrealize = gdm_greeter_panel_real_unrealize;
         widget_class->size_request = gdm_greeter_panel_real_size_request;
+        widget_class->show = gdm_greeter_panel_real_show;
+        widget_class->hide = gdm_greeter_panel_real_hide;
+
         signals[LANGUAGE_SELECTED] =
                 g_signal_new ("language-selected",
                               G_TYPE_FROM_CLASS (object_class),
@@ -391,6 +434,17 @@ gdm_greeter_panel_class_init (GdmGreeterPanelClass *klass)
                               G_TYPE_NONE,
                               1, G_TYPE_STRING);
 
+        signals[LAYOUT_SELECTED] =
+                g_signal_new ("layout-selected",
+                              G_TYPE_FROM_CLASS (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (GdmGreeterPanelClass, layout_selected),
+                              NULL,
+                              NULL,
+                              g_cclosure_marshal_VOID__STRING,
+                              G_TYPE_NONE,
+                              1, G_TYPE_STRING);
+
         signals[SESSION_SELECTED] =
                 g_signal_new ("session-selected",
                               G_TYPE_FROM_CLASS (object_class),
@@ -402,6 +456,16 @@ gdm_greeter_panel_class_init (GdmGreeterPanelClass *klass)
                               G_TYPE_NONE,
                               1, G_TYPE_STRING);
 
+        g_object_class_install_property (object_class,
+                                         PROP_MONITOR,
+                                         g_param_spec_int ("monitor",
+                                                           "Xinerama monitor",
+                                                           "The monitor (in terms of Xinerama) which the window is on",
+                                                           0,
+                                                           G_MAXINT,
+                                                           0,
+                                                           G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+
         g_type_class_add_private (klass, sizeof (GdmGreeterPanelPrivate));
 }
 
@@ -423,6 +487,26 @@ on_language_activated (GdmLanguageOptionWidget *widget,
         g_free (language);
 }
 
+static void
+on_layout_activated (GdmLayoutOptionWidget *widget,
+                     GdmGreeterPanel       *panel)
+{
+
+        char *layout;
+
+        layout = gdm_layout_option_widget_get_current_layout_name (GDM_LAYOUT_OPTION_WIDGET (panel->priv->layout_option_widget));
+
+        if (layout == NULL) {
+                return;
+        }
+
+        g_debug ("GdmGreeterPanel: activating selected layout %s", layout);
+        gdm_layout_activate (layout);
+
+        g_signal_emit (panel, signals[LAYOUT_SELECTED], 0, layout);
+
+        g_free (layout);
+}
 static void
 on_session_activated (GdmSessionOptionWidget *widget,
                       GdmGreeterPanel        *panel)
@@ -442,54 +526,21 @@ on_session_activated (GdmSessionOptionWidget *widget,
 }
 
 static void
-on_a11y_dialog_response (GtkDialog       *dialog,
-                         int              response,
-                         GdmGreeterPanel *panel)
+on_animation_tick (GdmGreeterPanel *panel,
+                   double           progress)
 {
-        g_signal_handlers_disconnect_by_func (dialog,
-                                              on_a11y_dialog_response,
-                                              panel);
+        panel->priv->progress = progress * log ((G_E - 1.0) * progress + 1.0);
 
-        gtk_widget_destroy (GTK_WIDGET (dialog));
-        panel->priv->a11y_dialog = NULL;
-}
-
-static void
-on_a11y_button_clicked (GtkButton       *button,
-                        GdmGreeterPanel *panel)
-{
-        if (panel->priv->a11y_dialog == NULL) {
-                panel->priv->a11y_dialog = gdm_a11y_preferences_dialog_new ();
-                g_signal_connect (panel->priv->a11y_dialog,
-                                  "response",
-                                  G_CALLBACK (on_a11y_dialog_response),
-                                  panel);
-        }
-        gtk_window_present (GTK_WINDOW (panel->priv->a11y_dialog));
-}
-
-static gboolean
-is_a11y_button_disabled (GdmGreeterPanel *panel)
-{
-        GConfClient *client;
-        GError      *error;
-        gboolean     disabled;
-
-        client = gconf_client_get_default ();
-        error = NULL;
-        disabled = gconf_client_get_bool (client, KEY_DISABLE_A11Y_BUTTON, &error);
-        if (error != NULL) {
-                g_debug ("GdmGreeterPanel: unable to get disable-accessibility-button configuration: %s", error->message);
-                g_error_free (error);
-        }
-        g_object_unref (client);
+        gtk_widget_queue_resize (GTK_WIDGET (panel));
 }
 
 static void
 gdm_greeter_panel_init (GdmGreeterPanel *panel)
 {
         NaTray    *tray;
-        GtkWidget *image;
+        GtkWidget *spacer;
+
+        gdm_profile_start (NULL);
 
         panel->priv = GDM_GREETER_PANEL_GET_PRIVATE (panel);
 
@@ -505,37 +556,48 @@ gdm_greeter_panel_init (GdmGreeterPanel *panel)
 
         gtk_window_set_keep_above (GTK_WINDOW (panel), TRUE);
         gtk_window_set_type_hint (GTK_WINDOW (panel), GDK_WINDOW_TYPE_HINT_DOCK);
-        gtk_window_set_opacity (GTK_WINDOW (panel), 0.75);
+        gtk_window_set_opacity (GTK_WINDOW (panel), 0.85);
 
         panel->priv->hbox = gtk_hbox_new (FALSE, 12);
         gtk_container_set_border_width (GTK_CONTAINER (panel->priv->hbox), 0);
         gtk_widget_show (panel->priv->hbox);
         gtk_container_add (GTK_CONTAINER (panel), panel->priv->hbox);
 
-        if (! is_a11y_button_disabled (panel)) {
-                panel->priv->a11y_button = gtk_button_new ();
-                image = gtk_image_new_from_icon_name ("preferences-desktop-accessibility", GTK_ICON_SIZE_BUTTON);
-                gtk_container_add (GTK_CONTAINER (panel->priv->a11y_button), image);
-                gtk_widget_show (image);
-                gtk_widget_show (panel->priv->a11y_button);
-                g_signal_connect (G_OBJECT (panel->priv->a11y_button),
-                                  "clicked",
-                                  G_CALLBACK (on_a11y_button_clicked), panel);
+        panel->priv->alignment = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
+        gtk_box_pack_start (GTK_BOX (panel->priv->hbox), panel->priv->alignment, TRUE, TRUE, 0);
+        gtk_widget_show (panel->priv->alignment);
 
-                gtk_box_pack_start (GTK_BOX (panel->priv->hbox), panel->priv->a11y_button, FALSE, FALSE, 0);
-        }
+        panel->priv->option_hbox = gtk_hbox_new (FALSE, 12);
+        gtk_widget_show (panel->priv->option_hbox);
+        gtk_container_add (GTK_CONTAINER (panel->priv->alignment), panel->priv->option_hbox);
 
+        spacer = gtk_label_new ("");
+        gtk_box_pack_start (GTK_BOX (panel->priv->option_hbox), spacer, TRUE, TRUE, 6);
+        gtk_widget_show (spacer);
+
+        gdm_profile_start ("creating option widget");
         panel->priv->language_option_widget = gdm_language_option_widget_new ();
         g_signal_connect (G_OBJECT (panel->priv->language_option_widget),
                           "language-activated",
                           G_CALLBACK (on_language_activated), panel);
-        gtk_box_pack_start (GTK_BOX (panel->priv->hbox), panel->priv->language_option_widget, FALSE, FALSE, 6);
+        gtk_box_pack_start (GTK_BOX (panel->priv->option_hbox), panel->priv->language_option_widget, FALSE, FALSE, 6);
+        gdm_profile_end ("creating option widget");
+
+        panel->priv->layout_option_widget = gdm_layout_option_widget_new ();
+        g_signal_connect (G_OBJECT (panel->priv->layout_option_widget),
+                          "layout-activated",
+                          G_CALLBACK (on_layout_activated), panel);
+        gtk_box_pack_start (GTK_BOX (panel->priv->option_hbox), panel->priv->layout_option_widget, FALSE, FALSE, 6);
 
         panel->priv->session_option_widget = gdm_session_option_widget_new ();
         g_signal_connect (G_OBJECT (panel->priv->session_option_widget),
                           "session-activated",
                           G_CALLBACK (on_session_activated), panel);
-        gtk_box_pack_start (GTK_BOX (panel->priv->hbox), panel->priv->session_option_widget, FALSE, FALSE, 6);
+        gtk_box_pack_start (GTK_BOX (panel->priv->option_hbox), panel->priv->session_option_widget, FALSE, FALSE, 6);
+
+        spacer = gtk_label_new ("");
+        gtk_box_pack_start (GTK_BOX (panel->priv->option_hbox), spacer, TRUE, TRUE, 6);
+        gtk_widget_show (spacer);
 
         /* FIXME: we should only show hostname on panel when connected
            to a remote host */
@@ -547,7 +609,7 @@ gdm_greeter_panel_init (GdmGreeterPanel *panel)
 
         panel->priv->clock = gdm_clock_widget_new ();
         gtk_box_pack_end (GTK_BOX (panel->priv->hbox),
-                          GTK_WIDGET (panel->priv->clock), FALSE, FALSE, 6);
+                            GTK_WIDGET (panel->priv->clock), FALSE, FALSE, 6);
         gtk_widget_show (panel->priv->clock);
 
         tray = na_tray_new_for_screen (gtk_window_get_screen (GTK_WINDOW (panel)),
@@ -556,6 +618,15 @@ gdm_greeter_panel_init (GdmGreeterPanel *panel)
         gtk_widget_show (GTK_WIDGET (tray));
 
         gdm_greeter_panel_hide_user_options (panel);
+
+        panel->priv->progress = 0.0;
+        panel->priv->animation_timer = gdm_timer_new ();
+        g_signal_connect_swapped (panel->priv->animation_timer,
+                                  "tick",
+                                  G_CALLBACK (on_animation_tick),
+                                  panel);
+
+        gdm_profile_end (NULL);
 }
 
 static void
@@ -570,15 +641,21 @@ gdm_greeter_panel_finalize (GObject *object)
 
         g_return_if_fail (greeter_panel->priv != NULL);
 
+        g_signal_handlers_disconnect_by_func (object, on_animation_tick, greeter_panel);
+        g_object_unref (greeter_panel->priv->animation_timer);
+
         G_OBJECT_CLASS (gdm_greeter_panel_parent_class)->finalize (object);
 }
 
 GtkWidget *
-gdm_greeter_panel_new (void)
+gdm_greeter_panel_new (GdkScreen *screen,
+                       int        monitor)
 {
         GObject *object;
 
         object = g_object_new (GDM_TYPE_GREETER_PANEL,
+                               "screen", screen,
+                               "monitor", monitor,
                                NULL);
 
         return GTK_WIDGET (object);
@@ -589,6 +666,7 @@ gdm_greeter_panel_show_user_options (GdmGreeterPanel *panel)
 {
         gtk_widget_show (panel->priv->session_option_widget);
         gtk_widget_show (panel->priv->language_option_widget);
+        gtk_widget_show (panel->priv->layout_option_widget);
 }
 
 void
@@ -596,12 +674,17 @@ gdm_greeter_panel_hide_user_options (GdmGreeterPanel *panel)
 {
         gtk_widget_hide (panel->priv->session_option_widget);
         gtk_widget_hide (panel->priv->language_option_widget);
+        gtk_widget_hide (panel->priv->layout_option_widget);
+
+        g_debug ("GdmGreeterPanel: activating default layout");
+        gdm_layout_activate (NULL);
 }
 
 void
 gdm_greeter_panel_reset (GdmGreeterPanel *panel)
 {
         gdm_greeter_panel_set_default_language_name (panel, NULL);
+        gdm_greeter_panel_set_default_layout_name (panel, NULL);
         gdm_greeter_panel_set_default_session_name (panel, NULL);
         gdm_greeter_panel_hide_user_options (panel);
 }
@@ -633,6 +716,28 @@ gdm_greeter_panel_set_default_language_name (GdmGreeterPanel *panel,
         g_free (normalized_language_name);
 }
 
+void
+gdm_greeter_panel_set_default_layout_name (GdmGreeterPanel *panel,
+                                           const char      *layout_name)
+{
+#ifdef HAVE_LIBXKLAVIER
+        g_return_if_fail (GDM_IS_GREETER_PANEL (panel));
+
+        if (layout_name != NULL &&
+            !gdm_option_widget_lookup_item (GDM_OPTION_WIDGET (panel->priv->layout_option_widget),
+                                            layout_name, NULL, NULL, NULL)) {
+                gdm_recent_option_widget_add_item (GDM_RECENT_OPTION_WIDGET (panel->priv->layout_option_widget),
+                                                   layout_name);
+        }
+
+        gdm_option_widget_set_default_item (GDM_OPTION_WIDGET (panel->priv->layout_option_widget),
+                                            layout_name);
+
+        g_debug ("GdmGreeterPanel: activating layout: %s", layout_name);
+        gdm_layout_activate (layout_name);
+#endif
+}
+
 void
 gdm_greeter_panel_set_default_session_name (GdmGreeterPanel *panel,
                                             const char      *session_name)
